# Minireq

The Minireq is a simple Command Line Interface (CLI) tool created for writing requirements in various Markdown files and building a general requirements document from these files (sort of GitBook). It also provides basic Domain Specific Languages (DSL) for tracing requirement to other software projects artifacts, as well as software estimations by PERT and FPA.

The Minireq is simple enough in itself, but when you combine it with Ruby (through Rake or Thor), a modern text editor (I use Atom) and Git, this combination will bring you unprecedented power and flexibility in requirements management.

## Promo

Minireq is supplied by Promo Minireq Project. This project contains software requirements for Minireq.

You can copy Promo Minireq Project content through `minireq promo` command. This command copies the content of promo project into the current directory.

When promo project is copied, you can simply create Minireq Requirements Document by `minireq doc` command.

## Installation

Install the minireq gem:

    $ gem install minireq

Of course you can choose the other standard way through cloning repository and installing the gem by hand (you must also have [Bundler](http://bundler.io/) installed):

    $ git clone https://github.com/nvoynov/minireq.git
    $ cd minireq    
    $ bundle install
    $ bundle exec rake install

## Usage

### Requirements file format

Requirements file is a usual Markdown file with a few additional compliances. Let's look at the next two example files `content.md` and `feature1.md`.

**content.md**
```markdown
# [f] Functional requirements
{{
suppress_id: true
child_order: feature1 feature2  
}}
# [i] Intefaces
# [n] Non-functional requirements
# [c] Design constraints
```

**feature1.md**
```markdown
# [f1] Feature 1
{{
parent: f
status: proposed
source: nvoynov  
}}
The System shall provide the Feature 1.

## [f1.data] Data
The System shall provide the next data for `Feature 1`

## [f1.cmd1] Command 1
The System shall provide `Command 1`. When `Command 1` is received, the System shall do something.

## [f2.cmd2] Command 2
The System shall provide `Command 2`. When `Command 2` is received, the System shall do something other than `Command 1` (see [[f2.cmd1]] for details).
```

Every requirement starts with markdown header. All the text between headers is a requirement body. The body can contain an excerpt with requirement attributes.

#### Attributes

The excerpt, the text in brackets `{{ }}`, contains attributes. You can place here anything you need, like requirement status, source, risk, priority, etc.

The next three attributes are **system attributes** and they  influence Minireq behavior. So that:
* `suppress_id: true` will suppress requirement [id] in header of the target document;
* `child_order: feature1 feature2` will sort child requirements in provided order;
* `parent: f` will place `[f1]` requirements subtree as a child requirement into `[f]` requirement.

All other attributes (`status`, `source`) are **user attributes** and do not influence Minireq behavior. These attributes are just held in requirements attributes hash.

#### Requirements tree

The Minireq will parse two files displayed above and build the next requirements tree:

```
[f] Functional requirements
    [f1] Feature 1
         [f1.data] Data
         [f1.cmd1] Command 1
         [f1.cmd2] Command 2
[i] Intefaces
[n] Non-functional requirements
[c] Design constraints
```

#### Check errors

Minireq checks requirements repository for duplicates and wrong links. To demonstrate these features, run the next commands (I hope you already have `content.md` and `feature.md` files) and see the output.

    $ cp reqs/feature1.md reqs/feature2.md
    $ minireq check

#### Create requirements document

Fix the errors by providing unique ids in `reqs/feature.2md`, then run `minireq doc`...

### CLI

You can see all available commands through `minireq help` command.
To start using Minireq just create a new project:

    $ minireq new requirements_project
    $ cd requirements_project

Or run `minireq init` to create a Minreq project structure.

    $ minireq init

Try minireq commands on Promo project:

    $ minireq promo
    $ minireq check
    $ minireq doc
    $ minireq trace create
    $ minireq pert create
    $ minireq fpa create

#### Extend it yourself!

You can and should extend standard CLI interface by your own commands. Just see example `creq.thor` file in the created project and add the necessary tasks. It is all Ruby code.

## Development

After checking out the repo, run `bin/setup` to install dependencies. Then, run `rake test` to run the tests. You can also run `bin/console` for an interactive prompt that will allow you to experiment.

To install this gem onto your local machine, run `bundle exec rake install`. To release a new version, update the version number in `version.rb`, and then run `bundle exec rake release`, which will create a git tag for the version, push git commits and tags, and push the `.gem` file to [rubygems.org](https://rubygems.org).

## Contributing

Bug reports and pull requests are welcome on GitHub at https://github.com/nvoynov/minireq.


## License

The gem is available as open source under the terms of the [MIT License](http://opensource.org/licenses/MIT).
